//==============================================
//  implied accidentals v0.1
//
//  Copyright (C)2012-2014 Jörn Eichler (heuchi) 
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//==============================================

import QtQuick 2.0
import MuseScore 1.0

MuseScore {
      version: "0.1"
      description: "This plugin adds implied accidentals"
      menuPath: "Plugins.Accidentals.Add Implied Accidentals"

      // if nothing is selected process whole score
      property bool processAll: false

      // function processNote
      function processNote(note,curKeySig) {
            // try to decide if the note is
            // outside the current key signature.
            // This doesn't with key sig changes yet
            // due to restrictions of the plugin framework.
            if((note.tpc > (curKeySig + 19))
             ||(note.tpc < (curKeySig + 13))) { 
                  // this note needs an accidental
                  // if there's none present anyway
                  if(note.accidental == null) {
                        // calculate type of needed accidental
                        var accidental=Accidental.NONE;
                        if(note.tpc < 6) {
                              accidental = Accidental.FLAT2;
                        } else if(note.tpc < 13) {
                              accidental = Accidental.FLAT;
                        } else if(note.tpc < 20) {
                              accidental = Accidental.NATURAL;
                        } else if(note.tpc < 27) {
                              accidental = Accidental.SHARP;
                        } else {
                              accidental = Accidental.SHARP2;
                        }
                        note.accidentalType = accidental;
                  }
            }
      }

      // function processTrack
      function processTrack(cursor,endTick,track) {
            var curKeySig = curScore.keysig;

            if(processAll) {
                  // we need to reset track first, otherwise
                  // rewind(0) doesn't work correctly
                  cursor.track=0; 
                  cursor.rewind(0);
            } else {
                  cursor.rewind(1);
            }

            var segment = cursor.segment;

            while(segment && (processAll || segment.tick < endTick)) {
                  // look for notes and grace notes
                  if(segment.elementAt(track) && segment.elementAt(track).type == Element.CHORD) {
                        // process graceNotes if present
                        if(segment.elementAt(track).graceNotes.length > 0) {
                              var graceChords = segment.elementAt(track).graceNotes;

                              for(var j=0;j<graceChords.length;j++) {
                                    var notes = graceChords[j].notes;
                                    for(var i=0;i<notes.length;i++) {
                                          processNote(notes[i],curKeySig);
                                    }
                              }
                        }
                        // process notes
                        var notes = segment.elementAt(track).notes;
                        
                        for(var i=0;i<notes.length;i++) {
                              processNote(notes[i],curKeySig);
                        }
                  }
                  segment=segment.next;
            }
      }

      function addAcc() {
            console.log("start add implied accidentals");

            if (typeof curScore === 'undefined' || curScore == null) {
                   console.log("error: no score!");	     
                   Qt.quit();
            }

            // find selection
            var startStaff;
            var endStaff;
            var endTick;
            
            var cursor = curScore.newCursor();
            cursor.rewind(1);
            if(!cursor.segment) {
                  // no selection
                  console.log("no selection: processing whole score");
                  processAll = true;
                  startStaff = 0;
                  endStaff = curScore.nstaves;
            } else {
                  startStaff = cursor.staffIdx;
                  cursor.rewind(2);
                  endStaff = cursor.staffIdx+1;
                  endTick = cursor.tick;
                  if(endTick == 0) {
                        // selection includes end of score
                        // calculate tick from last score segment
                        endTick = curScore.lastSegment.tick + 1;
                  }
                  cursor.rewind(1);
                  console.log("Selection is: Staves("+startStaff+"-"+endStaff+") Ticks("+cursor.tick+"-"+endTick+")");
            }      

            console.log("ProcessAll is "+processAll);

            for(var track=startStaff*4; track<endStaff*4; track++) {
                  processTrack(cursor,endTick,track);
            }

            console.log("end add implied accidentals");
            Qt.quit();
      }

      onRun: { 
            addAcc();
      }
}
